@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2003      Imendio HB
  * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2002-2003 Mikael Hallendal <micke@imendio.com>
- * Copyright (C) 2002-2003 CodeFactory AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,9 +31,9 @@
 #include "gossip-utils.h"
 #include "gossip-app.h"
 #include "gossip-chat.h"
-#include "gossip-group-chat.h"
 #include "disclosure-widget.h"
-
+#include "gossip-chat-view.h"
+#include "gossip-group-chat.h"
 
 /* Treeview columns */
 enum {
@@ -50,7 +50,7 @@ struct _GossipGroupChat {
 	LmMessageHandler *presence_handler;
 	
 	GtkWidget        *window;
-	GtkWidget        *textview;
+	GtkWidget        *text_view_sw;
 	GtkWidget        *input_entry;
 	GtkWidget        *input_text_view;
 	GtkWidget        *topic_entry;
@@ -60,12 +60,15 @@ struct _GossipGroupChat {
 	GtkWidget        *multi_vbox;
 	GtkWidget        *send_multi_button;
 
+	GossipChatView   *view;
+	
 	GossipJID        *jid;
 	gchar            *nick;
 
 	GCompletion      *completion;
 
-	GHashTable       *priv_chats;
+	GTimeVal          last_timestamp;
+	GList            *priv_chats;
 };
 
 typedef struct {
@@ -75,7 +78,6 @@ typedef struct {
 	GtkTreeIter      found_iter;
 } FindUserData;
 
-
 static void             destroy_group_chat                      (GossipGroupChat   *chat);
 static void             group_chat_window_destroy_cb            (GtkWidget         *widget,
 								 GossipGroupChat   *chat);
@@ -133,8 +135,8 @@ static GossipChat *     group_chat_priv_chat_new                (GossipGroupChat
 static void             group_chat_priv_chat_incoming           (GossipGroupChat   *chat,
 								 GossipJID         *jid,
 								 LmMessage         *m);
-static void             group_chat_priv_chat_destroyed          (GtkWidget         *dialog,
-								 GossipGroupChat   *chat);
+static void             group_chat_priv_chat_removed            (GossipGroupChat *chat,
+								 GossipChat   *priv_chat);
 static void             group_chat_send_multi_clicked_cb        (GtkWidget         *unused,
 								 GossipGroupChat   *chat);
 static void             group_chat_priv_chats_disconnect        (GossipGroupChat   *chat);
@@ -176,7 +178,7 @@ destroy_group_chat (GossipGroupChat *chat)
 	g_free (chat->nick);
 	g_free (chat);
 
-	g_hash_table_destroy (chat->priv_chats);
+	g_list_free (chat->priv_chats);
 }
 
 static void
@@ -223,9 +225,9 @@ group_chat_get_for_jid (GossipJID *jid)
 	
 	chat->jid = gossip_jid_ref (jid);
 	chat->inited = FALSE;
+	chat->last_timestamp.tv_sec = chat->last_timestamp.tv_usec = 0;
 
-	chat->priv_chats = g_hash_table_new_full (g_str_hash, g_str_equal,
-						  g_free, NULL);
+	chat->priv_chats = NULL;
 	
 	group_chat_create_gui (chat);
 
@@ -264,7 +266,7 @@ group_chat_create_gui (GossipGroupChat *chat)
 				       "group_chat_window",
 				       NULL,
 				       "group_chat_window", &chat->window,
-				       "chat_textview", &chat->textview,
+				       "chat_view_sw", &chat->text_view_sw,
 				       "input_entry", &chat->input_entry,
 				       "input_textview", &chat->input_text_view,
 				       "topic_entry", &chat->topic_entry,
@@ -282,12 +284,21 @@ group_chat_create_gui (GossipGroupChat *chat)
 			      "input_entry", "activate", group_chat_activate_cb,
 			      "input_entry", "key_press_event", group_chat_key_press_event_cb,
 			      "input_textview", "key_press_event", group_chat_key_press_event_cb,
-			      "chat_textview", "focus_in_event", group_chat_focus_in_event_cb,
 			      "topic_entry", "activate", group_chat_topic_activate_cb,
 			      "disclosure", "toggled", group_chat_disclosure_toggled_cb,
 			      "send_multi_button", "clicked", group_chat_send_multi_clicked_cb,
 			      NULL);
 
+	chat->view = gossip_chat_view_new ();
+	gtk_container_add (GTK_CONTAINER (chat->text_view_sw), 
+			   GTK_WIDGET (chat->view));
+	gtk_widget_show (GTK_WIDGET (chat->view));
+
+	g_signal_connect (chat->view,
+			  "focus_in_event",
+			  G_CALLBACK (group_chat_focus_in_event_cb),
+			  chat);
+			  
 	g_object_unref (glade);
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
@@ -312,12 +323,10 @@ group_chat_create_gui (GossipGroupChat *chat)
 	g_completion_set_compare (chat->completion,
 				  group_chat_completion_compare);
 		
-	gossip_text_view_setup_tags (GTK_TEXT_VIEW (chat->textview));
-
 	gtk_widget_grab_focus (chat->input_entry);
 	group_chat_setup_tree (chat);
 
-	gossip_text_view_set_margin (GTK_TEXT_VIEW (chat->textview), 3);
+	gossip_chat_view_set_margin (chat->view, 3);
 }
 
 GossipGroupChat *
@@ -416,7 +425,7 @@ group_chat_send (GossipGroupChat *chat,
 	else if (g_ascii_strcasecmp (msg, "/clear") == 0) {
 		GtkTextBuffer *buffer;
 
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->textview));
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->view));
 		gtk_text_buffer_set_text (buffer, "", -1);
 		
 		return;
@@ -430,6 +439,8 @@ group_chat_send (GossipGroupChat *chat,
 
 	lm_connection_send (connection, m, NULL);
 	lm_message_unref (m);
+
+	gossip_app_force_non_away ();
 }
 
 static void
@@ -464,7 +475,7 @@ group_chat_row_activated_cb (GtkTreeView       *view,
 			    COL_JID,  &jid,
 			    -1);
 
-	group_chat_priv_chat_new (chat, jid);
+	/* group_chat_priv_chat_new (chat, jid); */
 }
 
 static void
@@ -477,7 +488,7 @@ group_chat_setup_tree (GossipGroupChat *chat)
 	GtkTreeViewColumn *col;
 
 	tree = GTK_TREE_VIEW (chat->tree);
-	tv = GTK_TEXT_VIEW (chat->textview);
+	tv = GTK_TEXT_VIEW (chat->view);
 
 	gtk_tree_view_set_headers_visible (tree, FALSE);
 	
@@ -834,8 +845,8 @@ group_chat_message_handler (LmMessageHandler *handler,
 		node = lm_message_node_get_child (m->node, "body");
 		if (node) {
 			timestamp = gossip_utils_get_timestamp_from_message (m);
-			
-			gossip_text_view_append_chat_message (GTK_TEXT_VIEW (chat->textview),
+		
+			gossip_chat_view_append_chat_message (chat->view,
 							      timestamp,
 							      chat->nick,
 							      gossip_jid_get_resource (jid),
@@ -875,7 +886,7 @@ group_chat_presence_handler (LmMessageHandler *handler,
 	GdkPixbuf        *pixbuf;
 	GtkTreeIter       iter;
 	LmMessageNode    *node;
-	const gchar      *filename;
+	const gchar      *stock;
 
 	from = lm_message_node_get_attribute (m->node, "from");
 	jid = gossip_jid_new (from);
@@ -928,12 +939,15 @@ group_chat_presence_handler (LmMessageHandler *handler,
 	case LM_MESSAGE_SUB_TYPE_AVAILABLE:
 		node = lm_message_node_get_child (m->node, "show");
 		if (node) {
-			filename = gossip_utils_get_show_filename (node->value);
+			stock = gossip_get_icon_for_show_string (node->value);
 		} else {
-			filename = gossip_utils_get_show_filename (NULL);
+			stock = gossip_get_icon_for_show_string (NULL);
 		}
-
-		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
+		
+		pixbuf = gtk_widget_render_icon (chat->window,
+						 stock,
+						 GTK_ICON_SIZE_MENU,
+						 NULL);
 		
 		if (group_chat_find_user (chat, jid, &iter)) {
 			gtk_list_store_set (GTK_LIST_STORE (model),
@@ -1025,35 +1039,47 @@ group_chat_iter_compare_func (GtkTreeModel *model,
 }
 
 static GossipChat *
-group_chat_priv_chat_new (GossipGroupChat *chat, 
-			  GossipJID       *jid)
+group_chat_priv_chat_new (GossipGroupChat *chat, GossipJID *jid)
 {
-	GossipChat  *priv_chat;
-	const gchar *nick;
-	gchar       *key;
-	GtkWidget   *dialog;
+	GossipChat       *priv_chat = NULL;
+	GList            *l;
+	GossipRosterItem *item;
+/*	gchar            *name; */
 
-	nick = gossip_jid_get_resource (jid);
+	for (l = chat->priv_chats; l; l = l->next) {
+		GossipChat       *p_chat = GOSSIP_CHAT (l->data);
+		GossipRosterItem *item = gossip_chat_get_item (p_chat);
+		GossipJID        *j = gossip_roster_item_get_jid (item);
+		
+		if (gossip_jid_equals (j, jid)) {
+			priv_chat = p_chat;
+			break;
+		}
+	}
 	
-	priv_chat = g_hash_table_lookup (chat->priv_chats, nick);
 	if (priv_chat) {
 		return priv_chat;
 	}
+
+	item = gossip_roster_item_new (jid);
+	/* name = g_strdup_printf ("%s@%s", gossip_jid_get_resource (jid),
+				gossip_jid_get_part_name (jid));
+	gossip_roster_item_set_name (item, name);
+	g_free (name); */
+
+	priv_chat = gossip_chat_get_for_group_chat (item);
+	gossip_roster_item_unref (item);
+
+	chat->priv_chats = g_list_prepend (chat->priv_chats, priv_chat);
 	
-	priv_chat = gossip_chat_get_for_group_chat (jid);
+	g_object_weak_ref (G_OBJECT (priv_chat),
+			   (GWeakNotify) group_chat_priv_chat_removed,
+			   chat);
 
-	key = g_strdup (nick);
-	g_hash_table_insert (chat->priv_chats, key, priv_chat);
+	gossip_chat_present (priv_chat);
 
-	dialog = gossip_chat_get_dialog (priv_chat);
+	g_object_unref (priv_chat);
 	
-	g_object_set_data (G_OBJECT (dialog), "key", key);
-
-	g_signal_connect (dialog,
-			  "destroy", 
-			  G_CALLBACK (group_chat_priv_chat_destroyed),
-			  chat);
-
 	return priv_chat;
 }
 
@@ -1063,34 +1089,16 @@ group_chat_priv_chat_incoming (GossipGroupChat *chat,
 			       LmMessage       *m)
 {
 	GossipChat  *priv_chat;
-	const gchar *nick;
-	const gchar *from;
 
-	nick = gossip_jid_get_resource (jid);
-	priv_chat = g_hash_table_lookup (chat->priv_chats, nick);
-	
-	if (!priv_chat) {
-		GossipJID *jid;
+	priv_chat = group_chat_priv_chat_new (chat, jid);
 
-		from = lm_message_node_get_attribute (m->node, "from");
-		jid = gossip_jid_new (from);
-
-		priv_chat = group_chat_priv_chat_new (chat, jid);
-
-		gossip_jid_unref (jid);
-	}
-	
 	gossip_chat_append_message (priv_chat, m);
 }
 
 static void
-group_chat_priv_chat_destroyed (GtkWidget *dialog, GossipGroupChat *chat)
+group_chat_priv_chat_removed (GossipGroupChat *chat, GossipChat *priv_chat)
 {
-	gchar *key;
-	
-	key = (gchar *) g_object_get_data (G_OBJECT (dialog), "key");
-	
-	g_hash_table_remove (chat->priv_chats, key);
+	chat->priv_chats = g_list_remove (chat->priv_chats, priv_chat);
 }
 
 static void
@@ -1126,13 +1134,13 @@ set_status_foreach (gpointer key,
 	
 	lm_message_node_set_attributes (m->node, "to", key, NULL);
 	lm_connection_send (connection, m, NULL);
-};
+}
 
 void
-gossip_group_chat_set_status (GossipStatus status)
+gossip_group_chat_set_show (GossipShow show)
 {
 	LmMessage   *m;
-	const gchar *show;
+	const gchar *show_str;
 
 	group_chat_init ();
 	
@@ -1140,43 +1148,29 @@ gossip_group_chat_set_status (GossipStatus status)
 					  LM_MESSAGE_TYPE_PRESENCE,
 					  LM_MESSAGE_SUB_TYPE_AVAILABLE);
 	
-	show = gossip_status_to_string (status);
+	show_str = gossip_utils_show_to_string (show);
 	if (show) {
-		lm_message_node_add_child (m->node, "show", show);
+		lm_message_node_add_child (m->node, "show", show_str);
 	}
 
-	/*lm_message_node_add_child (m->node, "status", "Online");*/
+	/* FIXME set status string, lm_message_node_add_child (m->node, "status", "Online");*/
 	g_hash_table_foreach (group_chats, set_status_foreach, m);
 
 	lm_message_unref (m);
 }
 
-static void
-disconnect_foreach (gpointer key,
-		    gpointer value,
-		    gpointer data)
-{
-	GossipGroupChat *chat;
-	GossipChat      *priv_chat;
-	GtkWidget       *dialog;
-
-	chat = data;
-	priv_chat = value;
-	
-	dialog = gossip_chat_get_dialog (value);
-
-	/* FIXME: This should ideally just make the entries unsensitive. */
-	gtk_widget_set_sensitive (dialog, FALSE);
-
-	g_signal_handlers_disconnect_by_func (dialog,
-					      group_chat_priv_chat_destroyed,
-					      data);
-}
-
 static void
 group_chat_priv_chats_disconnect (GossipGroupChat *chat) 
 {
-	g_hash_table_foreach (chat->priv_chats,
-			      disconnect_foreach,
-			      chat);
-}
+	GList *l;
+
+	for (l = chat->priv_chats; l; l = l->next) {
+		GossipChat *priv_chat = GOSSIP_CHAT (l->data);
+		
+		gtk_widget_set_sensitive (gossip_chat_get_widget (priv_chat),
+					  FALSE);
+		g_object_weak_unref (G_OBJECT(priv_chat),
+				     (GWeakNotify) group_chat_priv_chat_removed,
+				     chat);
+	}
+}	
